@@ -36,6 +36,8 @@ import Prelude hiding (read, length)
 import Control.Monad
 import Control.Monad.Primitive
 
+import Data.Proxy
+
 import Data.Word
 import Data.Int
 import Data.Bits
@@ -51,31 +53,35 @@ import Data.Vector.Algorithms.Common
 
 import qualified Data.Vector.Algorithms.Insertion as I
 
+import Foreign.Storable
+
 -- | The methods of this class specify the information necessary to sort
 -- arrays using the default ordering. The name 'Lexicographic' is meant
 -- to convey that index should return results in a similar way to indexing
 -- into a string.
 class Lexicographic e where
-  -- | Given a representative of a stripe and an index number, this
-  -- function should determine whether to stop sorting.
-  terminate :: e -> Int -> Bool
+  -- | Computes the length of a representative of a stripe. It should take 'n'
+  -- passes to sort values of extent 'n'. The extent may not be uniform across
+  -- all values of the type.
+  extent    :: e -> Int
+
   -- | The size of the bucket array necessary for sorting es
-  size      :: e -> Int
+  size      :: Proxy e -> Int
   -- | Determines which bucket a given element should inhabit for a
   -- particular iteration.
   index     :: Int -> e -> Int
 
 instance Lexicographic Word8 where
-  terminate _ n = n > 0
-  {-# INLINE terminate #-}
+  extent _ = 1
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index _ n = fromIntegral n
   {-# INLINE index #-}
 
 instance Lexicographic Word16 where
-  terminate _ n = n > 1
-  {-# INLINE terminate #-}
+  extent _ = 2
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = fromIntegral $ (n `shiftR`  8) .&. 255
@@ -84,8 +90,8 @@ instance Lexicographic Word16 where
   {-# INLINE index #-}
 
 instance Lexicographic Word32 where
-  terminate _ n = n > 3
-  {-# INLINE terminate #-}
+  extent _ = 4
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = fromIntegral $ (n `shiftR` 24) .&. 255
@@ -96,8 +102,8 @@ instance Lexicographic Word32 where
   {-# INLINE index #-}
 
 instance Lexicographic Word64 where
-  terminate _ n = n > 7
-  {-# INLINE terminate #-}
+  extent _ = 8
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = fromIntegral $ (n `shiftR` 56) .&. 255
@@ -112,8 +118,8 @@ instance Lexicographic Word64 where
   {-# INLINE index #-}
 
 instance Lexicographic Word where
-  terminate _ n = n > 7
-  {-# INLINE terminate #-}
+  extent _ = sizeOf (0 :: Word)
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = fromIntegral $ (n `shiftR` 56) .&. 255
@@ -128,16 +134,16 @@ instance Lexicographic Word where
   {-# INLINE index #-}
 
 instance Lexicographic Int8 where
-  terminate _ n = n > 0
-  {-# INLINE terminate #-}
+  extent _ = 1
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index _ n = 255 .&. fromIntegral n `xor` 128
   {-# INLINE index #-}
 
 instance Lexicographic Int16 where
-  terminate _ n = n > 1
-  {-# INLINE terminate #-}
+  extent _ = 2
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = fromIntegral $ ((n `xor` minBound) `shiftR` 8) .&. 255
@@ -146,8 +152,8 @@ instance Lexicographic Int16 where
   {-# INLINE index #-}
 
 instance Lexicographic Int32 where
-  terminate _ n = n > 3
-  {-# INLINE terminate #-}
+  extent _ = 4
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = fromIntegral $ ((n `xor` minBound) `shiftR` 24) .&. 255
@@ -158,8 +164,8 @@ instance Lexicographic Int32 where
   {-# INLINE index #-}
 
 instance Lexicographic Int64 where
-  terminate _ n = n > 7
-  {-# INLINE terminate #-}
+  extent _ = 8
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = fromIntegral $ ((n `xor` minBound) `shiftR` 56) .&. 255
@@ -174,8 +180,8 @@ instance Lexicographic Int64 where
   {-# INLINE index #-}
 
 instance Lexicographic Int where
-  terminate _ n = n > 7
-  {-# INLINE terminate #-}
+  extent _ = sizeOf (0 :: Int)
+  {-# INLINE extent #-}
   size _ = 256
   {-# INLINE size #-}
   index 0 n = ((n `xor` minBound) `shiftR` 56) .&. 255
@@ -190,8 +196,8 @@ instance Lexicographic Int where
   {-# INLINE index #-}
 
 instance Lexicographic B.ByteString where
-  terminate b i = i >= B.length b
-  {-# INLINE terminate #-}
+  extent = B.length
+  {-# INLINE extent #-}
   size _ = 257
   {-# INLINE size #-}
   index i b
@@ -199,15 +205,43 @@ instance Lexicographic B.ByteString where
     | otherwise       = fromIntegral (B.index b i) + 1
   {-# INLINE index #-}
 
+instance (Lexicographic a, Lexicographic b) => Lexicographic (a, b) where
+  extent (a,b) = extent a + extent b
+  {-# INLINE extent #-}
+  size _ = size (Proxy :: Proxy a) `max` size (Proxy :: Proxy b)
+  {-# INLINE size #-}
+  index i (a,b)
+    | i >= extent a = index i b
+    | otherwise     = index i a
+  {-# INLINE index #-}
+
+instance (Lexicographic a, Lexicographic b) => Lexicographic (Either a b) where
+  extent (Left  a) = 1 + extent a
+  extent (Right b) = 1 + extent b
+  {-# INLINE extent #-}
+  size _ = size (Proxy :: Proxy a) `max` size (Proxy :: Proxy b)
+  {-# INLINE size #-}
+  index 0 (Left  _) = 0
+  index 0 (Right _) = 1
+  index n (Left  a) = index (n-1) a
+  index n (Right b) = index (n-1) b
+  {-# INLINE index #-}
+
+-- | Given a representative of a stripe and an index number, this
+-- function determines whether to stop sorting.
+terminate :: Lexicographic e => e -> Int -> Bool
+terminate e i = i >= extent e
+{-# INLINE terminate #-}
+
 -- | Sorts an array using the default ordering. Both Lexicographic and
 -- Ord are necessary because the algorithm falls back to insertion sort
 -- for sufficiently small arrays.
 sort :: forall e m v. (PrimMonad m, MVector v e, Lexicographic e, Ord e)
      => v (PrimState m) e -> m ()
-sort v = sortBy compare terminate (size e) index v
- where e :: e
-       e = undefined
-{-# INLINE sort #-}
+sort v = sortBy compare terminate (size p) index v
+ where p :: Proxy e
+       p = Proxy
+{-# INLINABLE sort #-}
 
 -- | A fully parameterized version of the sorting algorithm. Again, this
 -- function takes both radix information and a comparison, because the
